'-------------------------------------------------------------------------------
' Filename   : RC5_Tx1a.bas
' Purpose    : RC5 Remote Control Transmitter
' Author     : Ger langezaal
' Controller : AT90S2313
' Compiler   : BASCOM-AVR   Rev. 1.11a
'-------------------------------------------------------------------------------
'
' connections 3x4 Keypad:
'
' column 1     ------------------[470 Ohm]---> Pb.0
' column 2     |     ------------[470 Ohm]---> Pb.1
' column 3     |     |     ------[470 Ohm]---> Pb.2
'              |     |     |
'
' row 1        1     2     3  ---[470 Ohm]---> Pb.4
' row 2        4     5     6  ---[470 Ohm]---> Pb.5
' row 3        7     8     9  ---[470 Ohm]---> Pb.6
' row 4        *     0     #  ---[470 Ohm]---> Pb.7
'
' IR Led:    +5V <---[A Led K]---[220 Ohm]---> Pb.3
'
'-------------------------------------------------------------------------------
'
$regfile = "2313def.dat"
$crystal = 8000000

Dim Rc5_address As Byte
Dim Rc5_command As Byte
Dim Key As Byte
Dim Key_prev As Byte
Dim Rc5_bit_nr As Byte
Dim Toggle_bit As Boolean

' 36kHz carrier reload value
'Const T_oc1 = 55                          'value for 4MHz crystal
Const T_oc1 = 110                         'value for 8MHz crystal
'Const T_oc1 = 138                         'value for 10MHz crystal

Const Carrier_on = &B01000000             'IR 36kHz carrier on
Const Carrier_off = &B11000000            'IR 36kHz carrier off

Config Kbd = Portb                        'keyboard connected at Port b
Config Timer1 = Timer , Prescale = 1 , Compare A = Toggle , Clear Timer = 1
Timer1 = 0
Compare1a = T_oc1                         'Pb.3 = OC1 = IR carrier output 36 kHz

Declare Sub Logic_0
Declare Sub Logic_1

Tccr1a = Carrier_off
Toggle_bit = 0

'---[ set RC-5 System Address ]-------------------------------------------------

Rc5_address = 0                           'TV = 0
'Rc5_address = 5                          'VCR = 5
'Rc5_address = 20                         'CD = 20

'---[ main program loop ]-------------------------------------------------------

Do
  Key = Getkbd()                          'get keyboard value
  Waitms 1                                'small delay
  If Key <> 16 Then                       'test if key is pressed else key = 16
    If Key_prev <> Key Then Toggle_bit = Not Toggle_bit       'set toggle bit
    Rc5_command = Lookup(key , Rc5_commands)       'get RC-5 command

'---[ transmit Address and Command as 14 bit RC-5 code ]------------------------
    Ddrb.3 = 1                            'set OC1 (Pb.3) = IR carrier output
    Logic_1                               'first start bit
    Logic_1                               'second start bit
    If Toggle_bit = 0 Then Logic_0 Else Logic_1       'toggle bit

    For Rc5_bit_nr = 4 to 0 STEP -1          '5 bit address, msb first
      If Rc5_address.rc5_bit_nr = 0 Then Logic_0 Else Logic_1
    Next

    For Rc5_bit_nr = 5 To 0 step -1           '6 bit command, msb first
      If Rc5_command.rc5_bit_nr = 0 Then Logic_0 Else Logic_1
    Next

    Tccr1a = Carrier_off
    Waitms 87                             'frame gap delay
  End If
  Key_prev = Key                          'save last key value
Loop

End

'-------------------------------------------------------------------------------

Sub Logic_0
  Tccr1a = Carrier_on
  Waitus 883
  Tccr1a = Carrier_off
  Waitus 884
End Sub

Sub Logic_1
  Tccr1a = Carrier_off
  Waitus 883
  Tccr1a = Carrier_on
  Waitus 884
End Sub

Rc5_commands:
Data 1 , 2 , 3 , 0                        'column 4 not used  (3x4 keypad)
Data 4 , 5 , 6 , 0
Data 7 , 8 , 9 , 0
Data 10 , 11 , 12 , 0

'---[ Keyboard layout example ]-------------------------------------------------
'
' key   command      key   command      key   command
'   1 = 1              2 = 2              3 = 3
'   4 = 4              5 = 5              6 = 6
'   7 = 7              8 = 8              9 = 9
'   * = 10             0 = 11             # = 12
'
'-------------------------------------------------------------------------------
'
' for more RC-5 Remote Control info:
' http://users.pandora.be/nenya/electronics/rc5/index.htm
' http://www.ustr.net/infrared/infrared1.shtml
'
'-------------------------------------------------------------------------------